/*
 *  Copyright (C) 2002 - 2005 Tomasz Kojm <tkojm@clamav.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 */

#if HAVE_CONFIG_H
#include "clamav-config.h"
#endif

#ifdef CLAMUKO

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <pthread.h>

#include <clamav.h>

#include "server.h"
#include "others.h"
#include "cfgparser.h"
#include "dazukoio.h"
#include "clamuko.h"
#include "output.h"

#include <search.h>
#include <fcntl.h>

struct dazuko_access *acc;
dazuko_id_t *dazuko_id;

int compare(const void *pa, const void *pb) {
	if(*(int *)pa < *(int *)pb) return -1;
	if(*(int *)pa > *(int *)pb) return 1;
	return 0;
}

void clamuko_exit(int sig)
{

    logg("*Clamuko: clamuko_exit(), signal %d\n", sig);

    if(clamuko_scanning) {
        logg("*Clamuko: stopped while scanning %s\n", acc->filename);
        acc->deny = 0;
        dazukoReturnAccess_TS(dazuko_id,&acc); /* is it needed ? */
    }

    if(dazukoUnregister_TS(&dazuko_id))
    	logg("!Can't unregister with Dazuko\n");

    logg("Clamuko stopped.\n");

    pthread_exit(NULL);
}

void *clamukoth(void *arg)
{
	struct thrarg *tharg = (struct thrarg *) arg;
	sigset_t sigset;
	const char *virname;
    struct sigaction act;
	unsigned long mask = 0;
	const struct cfgstruct *pt;
	short int scan;
	int sizelimit = 0;
	int scanmodified = 0;
	struct stat sb;
    void *root=NULL;
    void *val;

    clamuko_scanning = 0;

    /* ignore all signals except SIGUSR1 */
    sigfillset(&sigset);
    sigdelset(&sigset, SIGUSR1);
    sigdelset(&sigset, SIGSEGV);
    pthread_sigmask(SIG_SETMASK, &sigset, NULL);
    act.sa_handler = clamuko_exit;
    sigfillset(&(act.sa_mask));
    sigaction(SIGUSR1, &act, NULL);
    sigaction(SIGSEGV, &act, NULL);

    /* register */
    if(dazukoRegister_TS(&dazuko_id,"ClamAV", "r+")) {
	logg("!Clamuko: Can't register with Dazuko\n");
	return NULL;
    } else
	logg("Clamuko: Correctly registered with Dazuko.\n");

    /* access mask */
    if(cfgopt(tharg->copt, "ClamukoScanOnOpen")->enabled) {
	logg("Clamuko: Scan-on-open mode activated.\n");
	mask |= DAZUKO_ON_OPEN;
    }
    if(cfgopt(tharg->copt, "ClamukoScanOnClose")->enabled) {
	logg("Clamuko: Scan-on-close mode activated.\n");
	mask |= DAZUKO_ON_CLOSE;
    }
    if(cfgopt(tharg->copt, "ClamukoScanOnExec")->enabled) {
	logg("Clamuko: Scan-on-exec mode activated.\n");
	mask |= DAZUKO_ON_EXEC;
    }

    if(cfgopt(tharg->copt, "ClamukoScanOnCloseModified")->enabled) {
        logg("Clamuko: Scan-on-close-modified mode activated.\n");
        mask |= DAZUKO_ON_OPEN;
        mask |= DAZUKO_ON_CLOSE;
        scanmodified = 1;
    }

    if(!mask) {
        logg("!Access mask is not configured properly.\n");
        dazukoUnregister_TS(&dazuko_id);
        return NULL;
    }

    if(dazukoSetAccessMask_TS(dazuko_id,mask)) {
        logg("!Clamuko: Can't set access mask in Dazuko.\n");
        dazukoUnregister();
        return NULL;
    }

    if((pt = cfgopt(tharg->copt, "ClamukoIncludePath"))->enabled) {
	while(pt) {
	    if((dazukoAddIncludePath_TS(dazuko_id,pt->strarg))) {
		logg("!Clamuko: Dazuko -> Can't include path %s\n", pt->strarg);
		dazukoUnregister_TS(&dazuko_id);
		return NULL;
	    } else
		logg("Clamuko: Included path %s\n", pt->strarg);

	    pt = (struct cfgstruct *) pt->nextarg;
	}
    } else {
	logg("!Clamuko: please include at least one path.\n");
	dazukoUnregister_TS(&dazuko_id);
	return NULL;
    }

    if((pt = cfgopt(tharg->copt, "ClamukoExcludePath"))->enabled) {
	while(pt) {
	    if((dazukoAddExcludePath_TS(dazuko_id,pt->strarg))) {
		logg("!Clamuko: Dazuko -> Can't exclude path %s\n", pt->strarg);
		dazukoUnregister_TS(&dazuko_id);
		return NULL;
	    } else
		logg("Clamuko: Excluded path %s\n", pt->strarg);

	    pt = (struct cfgstruct *) pt->nextarg;
	}
    }

    sizelimit = cfgopt(tharg->copt, "ClamukoMaxFileSize")->numarg;
    if(sizelimit)
	logg("Clamuko: Max file size limited to %d bytes.\n", sizelimit);
    else
	logg("Clamuko: File size limit disabled.\n");

    while(1) {

        if(dazukoGetAccess_TS(dazuko_id,&acc) == 0) {
            clamuko_scanning = 1;
            scan = 1;
    
/*            logg("Examining: %s id: %i\n",acc->filename,tharg->sid);*/
            /* If opened for writing, cache filename for scanning when closed */
            if (scanmodified){
                scan = 0;
//                 logg("Dazuko Event: Val is: %i\n",acc->event);
                switch (acc->event)
                {
                    case DAZUKO_ON_CLOSE:
                        val = tfind((void *)acc->filename, &root, compare);
//                         logg("Dazuko on Close: Val is: %i\n",val);
                        if (val != NULL){
                            scan = 1;
                            tdelete((void *)acc->filename, &root, compare);
                            logg("Deleting File from tree: %s id: %i\n",acc->filename,tharg->sid);
                        }
                        break;
                    case DAZUKO_ON_OPEN:
                        if (acc->set_flags){
                            if 	((acc->flags & O_RDWR) 
                                || (acc->flags & O_WRONLY) 
                                || (acc->flags & O_APPEND) 
                                || (acc->flags & O_SYNC) 
                                || (acc->flags & O_CREAT) 
                                || (acc->flags & O_TRUNC) 
                                || (acc->flags & O_EXCL)){
                                val = tsearch((void *)acc->filename, &root, compare);
                                if(val == NULL) break;
                                logg("Adding file to tree: %s id: %i\n",acc->filename,tharg->sid);
                            }else{
                                val = tfind((void *)acc->filename, &root, compare);
//                                 logg("Dazuko On Open: Val is: %i\n",val);
                                if (val != NULL){
                                    scan = 1;
                                }
                            }	
                        }else{
                            scan = 1;
                        }
                        break;
                    default:
                        scan = 1;
                        break;
        
                }
            }
    
            if(sizelimit) {
                stat(acc->filename, &sb);
                if(sb.st_size > sizelimit) {
                    scan = 0;
                    logg("*Clamuko: %s skipped (too big)\n", acc->filename);
                }
            }
    
            if(scan && cl_scanfile(acc->filename, &virname, NULL, tharg->engine, tharg->limits, tharg->options) == CL_VIRUS) {
                logg("Clamuko: %s: %s FOUND\n", acc->filename, virname);
                virusaction(acc->filename, virname, tharg->copt);
                acc->deny = 1;
            } else
                acc->deny = 0;
    
            if(dazukoReturnAccess_TS(dazuko_id,&acc)) {
                logg("!Can't return access to Dazuko.\n");
                logg("Clamuko stopped.\n");
                dazukoUnregister_TS(&dazuko_id);
                clamuko_scanning = 0;
                return NULL;
            }
    
            clamuko_scanning = 0;
        }
    }

    /* can't be ;) */
    return NULL;
}

#endif
